@@ -1,41 +1,4 @@
 
-## 'align_forecast' -----------------------------------------------------
-
-test_that("'align_forecast' works", {
-  set.seed(0)
-  data <- expand.grid(age = poputils::age_labels(type = "lt", max = 60),
-                      time = 2000:2005,
-                      sex = c("F", "M"))
-  data$deaths <- rpois(n = nrow(data), lambda = 100)
-  data$exposure <- 100
-  formula <- deaths ~ age * sex + age * time
-  mod <- mod_pois(formula = formula,
-                  data = data,
-                  exposure = exposure) |>
-                  set_prior(age:time ~ SVD_RW(HMD))
-  mod <- set_n_draw(mod, n = 10)
-  mod <- fit(mod)
-  components_est_unst <- components(mod, standardize = "none")
-  components_est_st <- components(mod, standardize = "terms")
-  components_forecast <- forecast_components(mod = mod,
-                                             components_est = components_est_unst,
-                                             labels_forecast = 2006:2007)
-  ans_obtained <- align_forecast(mod = mod,
-                                       comp_forecast = components_forecast,
-                                       comp_est_st = components_est_st,
-                                       comp_est_unst = components_est_unst,
-                                       labels_forecast = 2006:2007)
-  ans_expected <- components_forecast
-  ans_expected$.fitted[1:2] <- ans_expected$.fitted[1:2] +
-    (components_est_st$.fitted[24] - components_est_unst$.fitted[24])
-  ans_expected$.fitted[3:30] <- ans_expected$.fitted[3:30] +
-    rep(components_est_st$.fitted[125:138] - components_est_unst$.fitted[125:138], 2)
-  ans_expected$.fitted[31:36] <- ans_expected$.fitted[31:36] +
-    rep(components_est_st$.fitted[154:156] - components_est_unst$.fitted[154:156], 2)
-  expect_identical(ans_obtained, ans_expected)
-})
-
-
 ## 'forecast_ar' --------------------------------------------------------------
 
 test_that("'forecast_ar' works, n_by = 1", {
@@ -55,23 +18,23 @@ test_that("'forecast_ar' works, n_by = 1", {
   set.seed(1)
   ans_expected <- rep(ar_est[1], 6)
   ans_expected[1] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ar_est[3:5]),
+                                      mean = sum(rev(coef) * ar_est[3:5]),
                                       sd = sd)
   ans_expected[2] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * c(ar_est[4:5],
+                                      mean = sum(rev(coef) * c(ar_est[4:5],
                                                           ans_expected[1])),
                                       sd = sd)
   ans_expected[3] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * c(ar_est[5], ans_expected[1:2])),
+                                      mean = sum(rev(coef) * c(ar_est[5], ans_expected[1:2])),
                                       sd = sd)
   ans_expected[4] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ans_expected[1:3]),
+                                      mean = sum(rev(coef) * ans_expected[1:3]),
                                       sd = sd)
   ans_expected[5] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ans_expected[2:4]),
+                                      mean = sum(rev(coef) * ans_expected[2:4]),
                                       sd = sd)
   ans_expected[6] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ans_expected[3:5]),
+                                      mean = sum(rev(coef) * ans_expected[3:5]),
                                       sd = sd)
   expect_equal(ans_obtained, ans_expected)
 })
@@ -93,42 +56,42 @@ test_that("'forecast_ar' works, n_by = 2", {
   ans_expected <- rvec::rnorm_rvec(n = 12, n_draw = 10)
   set.seed(1)
   ans_expected[1] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ar_est[4:5]),
+                                      mean = sum(rev(coef) * ar_est[4:5]),
                                       sd = sd)
   ans_expected[2] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * c(ar_est[5],
+                                      mean = sum(rev(coef) * c(ar_est[5],
                                                           ans_expected[1])),
                                       sd = sd)
   ans_expected[3] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ans_expected[1:2]),
+                                      mean = sum(rev(coef) * ans_expected[1:2]),
                                       sd = sd)
   ans_expected[4] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ans_expected[2:3]),
+                                      mean = sum(rev(coef) * ans_expected[2:3]),
                                       sd = sd)
   ans_expected[5] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ans_expected[3:4]),
+                                      mean = sum(rev(coef) * ans_expected[3:4]),
                                       sd = sd)
   ans_expected[6] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ans_expected[4:5]),
+                                      mean = sum(rev(coef) * ans_expected[4:5]),
                                       sd = sd)
   ans_expected[7] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ar_est[9:10]),
+                                      mean = sum(rev(coef) * ar_est[9:10]),
                                       sd = sd)
   ans_expected[8] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * c(ar_est[10],
+                                      mean = sum(rev(coef) * c(ar_est[10],
                                                           ans_expected[7])),
                                       sd = sd)
   ans_expected[9] <- rvec::rnorm_rvec(n = 1,
-                                      mean = sum(coef * ans_expected[7:8]),
+                                      mean = sum(rev(coef) * ans_expected[7:8]),
                                       sd = sd)
   ans_expected[10] <- rvec::rnorm_rvec(n = 1,
-                                       mean = sum(coef * ans_expected[8:9]),
+                                       mean = sum(rev(coef) * ans_expected[8:9]),
                                        sd = sd)
   ans_expected[11] <- rvec::rnorm_rvec(n = 1,
-                                       mean = sum(coef * ans_expected[9:10]),
+                                       mean = sum(rev(coef) * ans_expected[9:10]),
                                        sd = sd)
   ans_expected[12] <- rvec::rnorm_rvec(n = 1,
-                                       mean = sum(coef * ans_expected[10:11]),
+                                       mean = sum(rev(coef) * ans_expected[10:11]),
                                        sd = sd)
   expect_equal(ans_obtained, ans_expected)
 })
@@ -136,7 +99,7 @@ test_that("'forecast_ar' works, n_by = 2", {
 
 ## 'forecast_ar_svd' ----------------------------------------------------------
 
-test_that("'forecast_ar_svd' works", {
+test_that("'forecast_ar_svd' works - con is 'none'", {
   set.seed(0)
   prior <- SVD_AR1(HMD)
   dimnames_term <- list(year = letters[1:5],
@@ -181,6 +144,51 @@ test_that("'forecast_ar_svd' works", {
   expect_equal(ans_obtained, ans_expected)
 })
 
+test_that("'forecast_ar_svd' works - con is 'by' (does not affect results)", {
+  set.seed(0)
+  prior <- SVD_AR1(HMD, con = "by")
+  dimnames_term <- list(year = letters[1:5],
+                        age = poputils::age_labels(type = "lt", max = 60))
+  var_time <- "year"
+  var_age <- "age"
+  var_sexgender <- "sex"
+  labels_forecast <- letters[6:11]
+  dimnames_forecast <- replace(dimnames_term, var_time, list(labels_forecast))
+  components <- vctrs::vec_rbind(tibble::tibble(term = "year:age",
+                                                component = "hyper",
+                                                level = c("coef", "sd"),
+                                                .fitted = rvec::runif_rvec(n = 2, n_draw = 10)),
+                                 tibble::tibble(term = "year:age",
+                                                component = "svd",
+                                                level = paste(paste0("comp", 1:3),
+                                                              rep(letters[1:5], each = 3),
+                                                              sep = "."),
+                                                .fitted = rvec::rnorm_rvec(n = 15, n_draw = 10)))
+  set.seed(1)
+  ans_obtained <- forecast_ar_svd(prior = prior,
+                                  dimnames_term = dimnames_term,
+                                  dimnames_forecast = dimnames_forecast,
+                                  var_time = var_time,
+                                  var_age = var_age,
+                                  var_sexgender = var_sexgender,
+                                  components = components,
+                                  labels_forecast = labels_forecast)
+  ans_expected <- rvec::rnorm_rvec(n = 18, n_draw = 10)
+  coef <- components$.fitted[components$level == "coef"]
+  sd <- components$.fitted[components$level == "sd"]
+  set.seed(1)
+  for (i in 1:3) {
+    ans_expected[i] <- rvec::rnorm_rvec(n = 1,
+                                          mean = coef * components$.fitted[14 + i],
+                                          sd = sd)
+    for (j in 1:5)
+      ans_expected[i + j * 3] <- rvec::rnorm_rvec(n = 1,
+                                                  mean = coef * ans_expected[i + (j-1) * 3],
+                                                  sd = sd)
+  }
+  expect_equal(ans_obtained, ans_expected)
+})
+
 
 ## 'forecast_components' ------------------------------------------------------
 
@@ -212,18 +220,17 @@ test_that("'forecast_components' works", {
 
 test_that("'forecast_lin' works with n_by = 1", {
   set.seed(0)
-  intercept <- rvec::rnorm_rvec(n = 1, n_draw = 10)
   slope <- rvec::rnorm_rvec(n = 1, n_draw = 10)
   sd <- rvec::runif_rvec(n = 1, n_draw = 10)
   matrix_along_by_est <- matrix(0:4, nr = 5)
   matrix_along_by_forecast <- matrix(0:5, nr = 6)
   set.seed(1)
-  ans_obtained <- forecast_lin(intercept = intercept,
-                               slope = slope,
+  ans_obtained <- forecast_lin(slope = slope,
                                sd = sd,
                                matrix_along_by_est = matrix_along_by_est,
                                matrix_along_by_forecast = matrix_along_by_forecast)
   set.seed(1)
+  intercept <- -1 * (5 + 1) * slope / 2
   ans_expected <- rvec::rnorm_rvec(n = 6,
                                    mean = intercept + 6:11 * slope,
                                    sd = sd)
@@ -232,18 +239,17 @@ test_that("'forecast_lin' works with n_by = 1", {
 
 test_that("'forecast_lin' works with  n_by = 2", {
   set.seed(0)
-  intercept <- rvec::rnorm_rvec(n = 2, n_draw = 10)
   slope <- rvec::rnorm_rvec(n = 2, n_draw = 10)
   sd <- rvec::runif_rvec(n = 1, n_draw = 10)
   matrix_along_by_est <- t(matrix(0:9, nr = 2))
   matrix_along_by_forecast <- t(matrix(0:11, nr = 2))
   set.seed(1)
-  ans_obtained <- forecast_lin(intercept = intercept,
-                               slope = slope,
+  ans_obtained <- forecast_lin(slope = slope,
                                sd = sd,
                                matrix_along_by_est = matrix_along_by_est,
                                matrix_along_by_forecast = matrix_along_by_forecast)
   set.seed(1)
+  intercept <- -0.5 * (5 + 1) * slope
   ans_expected <- c(rvec::rnorm_rvec(n = 6, mean = intercept[1] + 6:11 * slope[1], sd = sd),
                     rvec::rnorm_rvec(n = 6, mean = intercept[2] + 6:11 * slope[2], sd = sd))
   ans_expected <- ans_expected[c(1, 7, 2, 8, 3, 9, 4, 10, 5, 11, 6, 12)]
@@ -255,32 +261,30 @@ test_that("'forecast_lin' works with  n_by = 2", {
 
 test_that("'forecast_lin_trend' works with n_by = 1", {
   set.seed(0)
-  intercept <- rvec::rnorm_rvec(n = 1, n_draw = 10)
   slope <- rvec::rnorm_rvec(n = 1, n_draw = 10)
   matrix_along_by_est <- matrix(0:4, nr = 5)
   matrix_along_by_forecast <- matrix(0:5, nr = 6)
   set.seed(1)
-  ans_obtained <- forecast_lin_trend(intercept = intercept,
-                                     slope = slope,
+  ans_obtained <- forecast_lin_trend(slope = slope,
                                      matrix_along_by_est = matrix_along_by_est,
                                      matrix_along_by_forecast = matrix_along_by_forecast)
   set.seed(1)
+  intercept <- -0.5 * 6 * slope
   ans_expected <- intercept + 6:11 * slope
   expect_equal(ans_obtained, ans_expected)
 })
 
 test_that("'forecast_lin_trend' works with  n_by = 2, transposed", {
   set.seed(0)
-  intercept <- rvec::rnorm_rvec(n = 2, n_draw = 10)
   slope <- rvec::rnorm_rvec(n = 2, n_draw = 10)
   matrix_along_by_est <- t(matrix(0:9, nr = 2))
   matrix_along_by_forecast <- t(matrix(0:11, nr = 2))
   set.seed(1)
-  ans_obtained <- forecast_lin_trend(intercept = intercept,
-                                     slope = slope,
+  ans_obtained <- forecast_lin_trend(slope = slope,
                                      matrix_along_by_est = matrix_along_by_est,
                                      matrix_along_by_forecast = matrix_along_by_forecast)
   set.seed(1)
+  intercept <- -0.5 * 6 * slope
   ans_expected <- c(intercept[1] + 6:11 * slope[1], intercept[2] + 6:11 * slope[2])
   ans_expected <- ans_expected[c(1, 7, 2, 8, 3, 9, 4, 10, 5, 11, 6, 12)]
   expect_equal(ans_obtained, ans_expected)
@@ -569,9 +573,9 @@ test_that("'forecast_seasvary' works", {
 })
 
 
-## 'make_data_forecast' -------------------------------------------------------
+## 'make_data_forecast_labels' ------------------------------------------------
 
-test_that("'forecast_components' works", {
+test_that("'make_data_forecast_labels' works", {
   set.seed(0)
   data <- expand.grid(age = 0:9, time = 2000:2005, sex = c("F", "M"))
   data$deaths <- rpois(n = nrow(data), lambda = 100)
@@ -582,8 +586,8 @@ test_that("'forecast_components' works", {
                   data = data,
                   exposure = exposure)
   mod <- set_n_draw(mod, n = 10)
-  ans <- make_data_forecast(mod = mod,
-                            labels_forecast = 2006:2008)
+  ans <- make_data_forecast_labels(mod = mod,
+                                   labels_forecast = 2006:2008)
   expect_identical(names(ans), names(data))
   expect_true(all(is.na(ans$deaths)))
   expect_true(all(is.na(ans$exposure)))
@@ -594,6 +598,64 @@ test_that("'forecast_components' works", {
 })
 
 
+## 'make_data_forecast_newdata' -----------------------------------------------
+
+test_that("'make_data_forecast_newdata' works", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9, time = 2000:2005, sex = c("F", "M"))
+  data$deaths <- rpois(n = nrow(data), lambda = 100)
+  data$exposure <- 100
+  data$unused <- 33
+  formula <- deaths ~ age * sex + sex * time
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = exposure)
+  mod <- set_n_draw(mod, n = 10)
+  newdata <- make_data_forecast_labels(mod = mod,
+                                       labels_forecast = 2006:2008)
+  ans_obtained <- make_data_forecast_newdata(mod = mod, newdata = newdata)
+  expect_identical(ans_obtained, newdata)
+})
+
+test_that("'make_data_forecast_newdata' raises correct error with variables missing", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9, time = 2000:2005, sex = c("F", "M"))
+  data$deaths <- rpois(n = nrow(data), lambda = 100)
+  data$exposure <- 100
+  data$unused <- 33
+  formula <- deaths ~ age * sex + sex * time
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = exposure)
+  mod <- set_n_draw(mod, n = 10)
+  newdata <- make_data_forecast_labels(mod = mod,
+                                       labels_forecast = 2006:2008)
+  newdata <- newdata[-1]
+  expect_error(make_data_forecast_newdata(mod = mod, newdata = newdata),
+               "Variable in model but not in `newdata`: \"age\".")
+  newdata <- newdata[-1]
+  expect_error(make_data_forecast_newdata(mod = mod, newdata = newdata),
+               "Variables in model but not in `newdata`: \"age\" and \"time\".")
+})
+
+test_that("'make_data_forecast_newdata' raises correct error when periods overlap", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9, time = 2000:2005, sex = c("F", "M"))
+  data$deaths <- rpois(n = nrow(data), lambda = 100)
+  data$exposure <- 100
+  data$unused <- 33
+  formula <- deaths ~ age * sex + sex * time
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = exposure)
+  mod <- set_n_draw(mod, n = 10)
+  newdata <- make_data_forecast_labels(mod = mod,
+                                       labels_forecast = 2005:2008)
+  expect_error(make_data_forecast_newdata(mod = mod, newdata = newdata),
+               "Time periods in `newdata` and `data` overlap.")
+})
+
+
 ## 'make_dimnames_terms_forecast' ---------------------------------------------
 
 test_that("'make_dimnames_terms_forecast' works - time_only is FALSE", {
